"use strict";

chrome.webRequest.onHeadersReceived.addListener(details => {
    let originUrl = details.originUrl || details.initiator;
    if (!originUrl || !originUrl.startsWith(location.origin)) {
        console.log(`Ignoring request from unrelated origin: ${originUrl}`);
        return;
    }
    let csp = new URL(details.url).searchParams.get("csp");
    if (!csp) {
        console.log(`Ignoring request without CSP override: ${details.url}`);
        return;
    }
    let { responseHeaders } = details;
    responseHeaders = responseHeaders.filter(h => {
        if (
            h.name.toLowerCase() === "x-frame-options" ||
            h.name.toLowerCase() === "content-security-policy"
        ) {
            console.log(`Removing ${h.name} : ${h.value}`);
            return false;
        }
        return true;
    });
    responseHeaders.push({
        name: "content-security-policy",
        value: csp,
    });
    console.log("Changing CSP to " + csp);
    return { responseHeaders };
}, {
    urls: ["*://*/*csp=*"],
    types: ["sub_frame"],
}, [
    "blocking",
    "responseHeaders",
]);

var outputContainer = document.getElementById("outputContainer");

function loadTestFrame(baseUrl, csp) {
    var url = new URL(baseUrl);
    url.searchParams.set("csp", csp);

    var div = document.createElement("div");

    div.appendChild(document.createElement("h2")).textContent = csp;

    div.appendChild(document.createElement("iframe")).src = url.href;

    outputContainer.append(div);
}
function startTest(url) {
    outputContainer.appendChild(document.createElement("h1")).textContent = "URL in frame: " + url;
    [
        "frame-ancestors 'none'",
        "frame-ancestors *",
        "frame-ancestors moz-extension:",
        "frame-ancestors " + location.origin,
    ].forEach(csp => loadTestFrame(url, csp));
}

document.forms.testForm.onsubmit = e => {
    e.preventDefault();
    startTest(document.forms.testForm.elements.url.value);
};
document.getElementById("clearButton").onclick = () => {
    outputContainer.textContent = "";
};

{
    let url = new URLSearchParams(location.search).get("url");
    if (url) {
        document.forms.testForm.elements.url.value = url;
    }
}
